Boomkinator = LibStub("AceAddon-3.0"):NewAddon("Boomkinator", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

local L    = LibStub("AceLocale-3.0"):GetLocale("Boomkinator")

local settings
local UnitBuff = _G.UnitBuff
local UnitDebuff = _G.UnitDebuff
local GetSpellInfo = _G.GetSpellInfo
local GetSpellCooldown= _G.GetSpellCooldown
local sformat = string.format

local hasSunfire = 0
local hasStars = 0
local timer = false

-- buttons
local bframe, bisIcon, bisTime, bmfIcon, bmfTime, bwrtIcon, bstfIcon, bstsIcon, bstsTime, bfonIcon, bfonTime, bsflIcon, bsflTime
local mfName, mfIcon, sfName, sfIcon
local bLastCycle

Boomkinator.Credits1 = "Boomkinator - by Aznamir (Lightbringer US)";

-------------------------------------------------------------------
-- Spell Settings
-------------------------------------------------------------------
Boomkinator.Spells = {
	[1] = GetSpellInfo(5570),  -- Insect Swarm
	[2] = GetSpellInfo(8921),  -- Moonfire
	[3] = GetSpellInfo(93402), -- Sunfire
	[4] = GetSpellInfo(5176),  -- Wrath
	[5] = GetSpellInfo(2912),  -- Starfire
	[6] = GetSpellInfo(78674), -- Starsurge
	[7] = GetSpellInfo(33831), -- Treants
	[8] = GetSpellInfo(50288), -- Starfall
	[9] = GetSpellInfo(48518), --eclipse lunar
	[10]= GetSpellInfo(48517), --eclipse solar
	[11]= GetSpellInfo(93400), -- shooting stars
};
-------------------------------------------------------------------
-- Interface settings
-------------------------------------------------------------------
Boomkinator.options =
{
	type = "group",
	name = "Boomkinator",
	args = {
		desc = {
			type = "description",
			order = 0,
			name = L["MOVE_DESC"],
		},
		reset = {
			type = "execute",
			order = 2,
			name = L["RESET"],
			desc = L["RESET_DESC"],
			func = function() Boomkinator:Reset() end,
		},
		scale = {
			type = "range",
			order = 1,
			name = L["BSC"],
			desc = L["BSC_DESC"],
			min = 0.4,
			max = 1.5,
			step = 0.05,
			get = function(info) return settings.buffscale end,
			set = function(info, val)
					settings.buffscale = val
					Boomkinator:UpdateLayout()
				end,
		},

	},
}
-------------------------------------------------------------------
-- Default Settings
-------------------------------------------------------------------
Boomkinator.defaults = {
	profile = {
		buffscale = 0.9,
		disabled = false,
	}
}
-------------------------------------------------------------------
-- Ace Framework Events
-------------------------------------------------------------------
function Boomkinator:OnInitialize()
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99"..Boomkinator.Credits1.."|r")
	self.db = LibStub("AceDB-3.0"):New("BoomkinatorDB", Boomkinator.defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	settings = self.db.profile
	Boomkinator.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Boomkinator", Boomkinator.options, "boom")
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Boomkinator", "Boomkinator")
	self:CreateLayout()
end

function Boomkinator:OnProfileChanged()
	settings = self.db.profile
	Boomkinator:UpdateLayout()
end

function Boomkinator:OnEnable()
	settings.disabled = false
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:UpdateLayout()
end

function Boomkinator:OnDisable()
	-- events
	settings.disabled = true
	self:UpdateLayout()
end

function Boomkinator:MediaUpdate()

end
-------------------------------------------------------------------
-- Service Functions
-------------------------------------------------------------------
function Boomkinator:FormatTime(time)
	if not time or time < 0 or time == 9999 then
		return ""
	end
	local mins = floor(time / 60)
	local secs = time - (mins * 60)
	return sformat("%d:%02d", mins, secs)
end

-------------------------------------------------------------------
-- Command Prompt Response
-------------------------------------------------------------------
function Boomkinator:Reset()
	local h = _G["BoomkinatorFrame"]
	h:ClearAllPoints()
	h:SetPoint("CENTER", "UIParent", "CENTER", 0, 0)
	--self:UpdateLayout()
end

-------------------------------------------------------------------
-- Internal Functions
-------------------------------------------------------------------

-------------------------------------------------------------------
-- External Event Handling
------------------------------------------------------------------
function Boomkinator:PLAYER_REGEN_ENABLED()
	if timer then
		self:CancelTimer(self.UpdateTimer)
		timer = false
	end
	self:HideLayout()
end

function Boomkinator:PLAYER_REGEN_DISABLED()
	local _, class = UnitClass("player")
	if (class == "DRUID") and select(5, GetTalentInfo(1,20)) == 1 then -- moonkin
		hasSunfire = select(5, GetTalentInfo(1,16))
		hasStars = select(5, GetTalentInfo(1,10))
		self.UpdateTimer = self:ScheduleRepeatingTimer("ButtonsUpdate", 0.1)
		timer = true
		self:ShowLayout()
	end
end
-------------------------------------------------------------------
-- Buff UI
-------------------------------------------------------------------
function Boomkinator:CreateLayout()
	bframe   = _G["BoomkinatorFrame"]
	bisIcon  = _G["BoomkinatorFrameIS"]
	bisTime  = _G["BoomkinatorFrameISTime"]
	bmfIcon  = _G["BoomkinatorFrameMF"]
	bmfTime  = _G["BoomkinatorFrameMFTime"]
	bwrtIcon = _G["BoomkinatorFrameWRT"]
	bstfIcon = _G["BoomkinatorFrameSTF"]
	bstsIcon = _G["BoomkinatorFrameSTS"]
	bstsTime = _G["BoomkinatorFrameSTSTime"]
	bfonIcon = _G["BoomkinatorFrameFON"]
	bfonTime = _G["BoomkinatorFrameFONTime"]
	bsflIcon = _G["BoomkinatorFrameSFL"]
	bsflTime = _G["BoomkinatorFrameSFLTime"]
	Boomkinator:UpdateLayout()
end

function Boomkinator:UpdateLayout()
	if InCombatLockdown() then return false end
	BoomkinatorFrame:SetScale(settings.buffscale)
	mfName, _, mfIcon = GetSpellInfo(8921) -- Moonfire
	sfName, _, sfIcon = GetSpellInfo(93402) -- Sunfire
	bLastCycle = 0
	Boomkinator:HideLayout()
end

function Boomkinator:ShowLayout()
	bframe:Show()
end

function Boomkinator:HideLayout()
	bframe:Hide()
end

function Boomkinator:GetSeverityColor(percent)
	if (percent >= 0.5) then
		return (1.0-percent)*2, 1.0, 0.0
	else
		return 1.0, percent*2, 0.0
	end
end

function Boomkinator:ButtonsUpdate()
	local expire
	local mooncycle = 0
	local suncycle  = 0
	local spellName, spellIcon
	local stars = 0
	-- show Moonfire or Sunfire

	local mfDuration, mfExpire, sfDuration, sfExpire

	spellName = Boomkinator.Spells[9] -- lunar
	local buffName, _, _, _, _, buffDuration, buffExpire = UnitBuff("player", spellName)
	if buffName then
		mooncycle = 1
		bLastCycle = 1
	end

	spellName = Boomkinator.Spells[10] -- solar
	buffName, _, _, _, _, buffDuration, buffExpire = UnitBuff("player", spellName)
	if buffName then
		suncycle = 1
		bLastCycle = 2
	end
	
	spellName = Boomkinator.Spells[11] -- shooting stars
	buffName, _, _, _, _, buffDuration, buffExpire = UnitBuff("player", spellName)
	if buffName then
		stars = 1
	end
	
	if UnitExists("target") then
		-- checking for debuffs on the target
		
		-- insect swarm
		local spellName = Boomkinator.Spells[1]
		_, _, _, _, _, buffDuration, buffExpire = UnitDebuff("target", spellName)

		if buffExpire then
			expire = buffExpire - GetTime()
			bisTime:SetText(Boomkinator:FormatTime(expire))
			bisIcon:SetVertexColor(1, 1, 1, 0.3)
			-- icon is permanent
		else
			bisTime:SetText("")
			bisIcon:SetVertexColor(1, 1, 1, 1)
			-- icon is permanent
		end

		-- moonfire and sunfire
		_, _, _, _, _, mfDuration, mfExpire = UnitDebuff("target", mfName)
		_, _, _, _, _, sfDuration, sfExpire = UnitDebuff("target", sfName)

		if mooncycle == 0 and suncycle == 0 then
			-- in the middle of cycles
			if sfExpire then -- sunfire still active
				expire = sfExpire - GetTime()
				bmfTime:SetText(Boomkinator:FormatTime(expire))
				bmfIcon:SetVertexColor(1, 1, 1, 0.3)
				bmfIcon:SetTexture(sfIcon)
			else
				if mfExpire then -- moonfire still active
					expire = mfExpire - GetTime()
					bmfTime:SetText(Boomkinator:FormatTime(expire))
					bmfIcon:SetVertexColor(1, 1, 1, 0.3)
				else -- moonfire inactive
					bmfTime:SetText("")
					bmfIcon:SetVertexColor(1, 1, 1, 1)
				end
				bmfIcon:SetTexture(mfIcon)
			end
		else
			-- moonfire or sunfire active
			if suncycle == 0 or hasSunfire == 0 then
				-- moonfire
				spellName, _, spellIcon = GetSpellInfo(8921) -- Moonfire
			else
				-- sunfire
				spellName, _, spellIcon = GetSpellInfo(93402) -- Sunfire
			end
			_, _, _, _, _, buffDuration, buffExpire = UnitDebuff("target", spellName)
			
			if buffExpire then
				expire = buffExpire - GetTime()
				bmfTime:SetText(Boomkinator:FormatTime(expire))
				bmfIcon:SetTexture(spellIcon)
				bmfIcon:SetVertexColor(1, 1, 1, 0.3)
			else
				bmfTime:SetText("")
				bmfIcon:SetTexture(spellIcon)
				bmfIcon:SetVertexColor(1, 1, 1, 1)
			end
		end
	end
	
	-- wrath or stafire
	if mooncycle == 1 then
		bwrtIcon:SetVertexColor(1, 1, 1, 0.3)
		bstfIcon:SetVertexColor(1, 1, 1, 1)
	elseif suncycle == 1 then
		bwrtIcon:SetVertexColor(1, 1, 1, 1)
		bstfIcon:SetVertexColor(1, 1, 1, 0.3)
	else
		if bLastCycle == 1 then -- from moon to sun
			bwrtIcon:SetVertexColor(1, 1, 1, 0.3)
			bstfIcon:SetVertexColor(1, 1, 1, 1)
		elseif bLastCycle == 2 then -- from Sun to Moon
			bwrtIcon:SetVertexColor(1, 1, 1, 1)
			bstfIcon:SetVertexColor(1, 1, 1, 0.3)
		else -- unknown transition
			bwrtIcon:SetVertexColor(1, 1, 1, 0.75)
			bstfIcon:SetVertexColor(1, 1, 1, 0.75)
		end
	end
	
	-- check cooldowns
	local start, duration
	-- starsurge
	spellName = Boomkinator.Spells[6]
	start, duration = GetSpellCooldown(spellName)

	if start == 0 or (start > 0 and duration < 1.5 ) then
		bstsTime:SetText("")
		if (mooncycle == 1 or suncycle == 1) and stars == 0 then
			-- spell is ready, but no need to cast it yet as we are in an active cycle
			bstsIcon:SetVertexColor(1, 1, 1, 0.3)
		else
			-- spell just reset because of shooting starts or spell is ready and need to cast to advance into next cycle
			bstsIcon:SetVertexColor(1, 1, 1, 1)
		end
	else
	-- on cooldown
		expire = start + duration - GetTime()
		bstsTime:SetText(Boomkinator:FormatTime(expire))
		bstsIcon:SetVertexColor(1, 1, 1, 0.3)
	end

	-- treants
	spellName = Boomkinator.Spells[7]
	start, duration = GetSpellCooldown(spellName)

	if start == 0 or (start > 0 and duration < 1.5 ) then
		bfonTime:SetText("")
		bfonIcon:SetVertexColor(1, 1, 1, 1)
	else
	-- on cooldown
		expire = start + duration - GetTime()
		bfonTime:SetText(Boomkinator:FormatTime(expire))
		bfonIcon:SetVertexColor(1, 1, 1, 0.3)
	end

	-- starfall
	spellName = Boomkinator.Spells[8]
	start, duration = GetSpellCooldown(spellName)
	if start == 0 or (start > 0 and duration < 1.5 ) then
		bsflTime:SetText("")
		bsflIcon:SetVertexColor(1, 1, 1, 1)
	else
	-- on cooldown
		expire = start + duration - GetTime()
		bsflTime:SetText(Boomkinator:FormatTime(expire))
		bsflIcon:SetVertexColor(1, 1, 1, 0.3)
	end
end
